﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Controllers;
using RESTWebAPI.Models;


namespace RESTWebAPI.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class ProductsController : ApiController
    {
        #region GET
        // READ
        // GET /api/products
        public IQueryable<MyProduct> Get()
        {
            var entities = new ProductsEntities();

            return from product in entities.Products
                   select new MyProduct
                       {
                           ID = product.ID,
                           Name = product.Name,
                           Price = product.Price,
                           Quantity = product.Quantity,
                           ReleaseDate = product.ReleaseDate
                       };
        }

        // GET /api/products/id
        public MyProduct Get(int id)
        {
            var entities = new ProductsEntities();
            MyProduct product =  (from p in entities.Products where p.ID == id
                                select new MyProduct
                                {
                                    ID = p.ID,
                                    Name = p.Name,
                                    Price = p.Price,
                                    Quantity = p.Quantity,
                                    ReleaseDate = p.ReleaseDate
                                }).First();
            if (product == null)
                throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.NotFound));
            return product;
        }
        #endregion GET

        #region POST
        // CREATE
        // POST /api/products
        public HttpResponseMessage Post(IEnumerable<MyProduct> products)
        {
            var entities = new ProductsEntities();
            if (ModelState.IsValid)
            {

                // detect conflicts
                foreach (MyProduct p in products)
                {
                    if (entities.Products.Where(e => e.ID == p.ID).Count() > 0)
                        throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                }

                //persist through EF
                foreach (MyProduct myProd in products)
                {
                    Product p = new Product();
                    p.ID = myProd.ID;
                    p.Name = myProd.Name;
                    p.Price = myProd.Price;
                    p.Quantity = myProd.Quantity;
                    p.ReleaseDate = myProd.ReleaseDate;
                    entities.Products.AddObject(p);
                }
                try
                {
                    entities.SaveChanges();
                }
                catch
                {
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                }
                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, products);
                // 201 Created should also return the location of the new resource.
                // As we are creating more than one we'll only add the location of the first.
                string uri = Url.Route(null, new { id = products.First<MyProduct>().ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region PUT
        // PUT /api/products/5
        public HttpResponseMessage Put(int id, MyProduct updatedProduct)
        {
            var entities = new ProductsEntities();
            if (ModelState.IsValid)
            {
                // find product
                Product p = entities.Products.Where(o => o.ID == updatedProduct.ID).First();
                if (p != null)
                {
                    p.ID = updatedProduct.ID;
                    p.Name = updatedProduct.Name;
                    p.Price = updatedProduct.Price;
                    p.Quantity = updatedProduct.Quantity;
                    p.ReleaseDate = updatedProduct.ReleaseDate;
                    try
                    {
                        entities.SaveChanges();
                        /* Alternatively the response can be
                         * 200 OK if update was enacted correctly (same as 204 No Content)
                         * The response does not have to include a representation of the entity updated
                        return new HttpResponseMessage(HttpStatusCode.OK);
                         */
                        return new HttpResponseMessage(HttpStatusCode.NoContent);
                    }
                    catch 
                    {
                        throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                    }
                }
                // if resource is not found it should be created
                p = new Product();
                p.ID = updatedProduct.ID;
                p.Name = updatedProduct.Name;
                p.Quantity = updatedProduct.Quantity;
                p.Price = updatedProduct.Price;
                p.ReleaseDate = updatedProduct.ReleaseDate;
                entities.Products.AddObject(p);
            
                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, updatedProduct);
                // 201 Created should also return the location of the new resource.
                string uri = Url.Route(null, new { id = updatedProduct.ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }

        // PUT /api/products/?ID=5&ID=6&ID=7
        public HttpResponseMessage Put([FromUri]IEnumerable<int> index, IEnumerable<MyProduct> updatedProducts)
        {
            var entities = new ProductsEntities();
            List<MyProduct> createProducts = new List<MyProduct>();
            bool created = false;
            if (ModelState.IsValid)
            {
                foreach (MyProduct updatedProduct in updatedProducts)
                {
                    Product p = entities.Products.Where(o => o.ID == updatedProduct.ID).First();
                    if (p != null)
                    {
                        p.ID = updatedProduct.ID;
                        p.Name = updatedProduct.Name;
                        p.Price = updatedProduct.Price;
                        p.Quantity = updatedProduct.Quantity;
                        p.ReleaseDate = updatedProduct.ReleaseDate;
                    }
                    else
                    {
                        created = true;
                        // if resource is not found it should be created
                        p = new Product();
                        p.ID = updatedProduct.ID;
                        p.Name = updatedProduct.Name;
                        p.Quantity = updatedProduct.Quantity;
                        p.Price = updatedProduct.Price;
                        p.ReleaseDate = updatedProduct.ReleaseDate;
                        entities.Products.AddObject(p);
                        createProducts.Add(updatedProduct);
                    }
                }
                try
                {
                    entities.SaveChanges();
                    /* Alternatively the response can be
                     * 200 OK if update was enacted correctly (same as 204 No Content)
                     * The response does not have to include a representation of the entity updated
                    return new HttpResponseMessage(HttpStatusCode.OK);
                     */
                    if (!created)
                    {
                        return new HttpResponseMessage(HttpStatusCode.NoContent);
                    }
                }
                catch
                {
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                }
                // we had to create entities
                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, createProducts);
                // 201 Created should also return the location of the new resource.
                string uri = Url.Route(null, new { id = createProducts.First().ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region DELETE
        // DELETE /api/products/5
        public HttpResponseMessage Delete(int id)
        {
            var entities = new ProductsEntities();
            if (ModelState.IsValid)
            {
                MyProduct deletedProduct = new MyProduct();
                Product entity = entities.Products.Where(p => p.ID == id).First();
                deletedProduct.ID = entity.ID;
                deletedProduct.Name = entity.Name;
                deletedProduct.Price = entity.Price;
                deletedProduct.Quantity = entity.Quantity;
                deletedProduct.ReleaseDate = entity.ReleaseDate;
                try
                {
                    entities.Products.DeleteObject(entity);
                    entities.SaveChanges();
                } catch 
                {
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                }
                return Request.CreateResponse(HttpStatusCode.OK, deletedProduct);
                /* Alternatively Delete can also return 
                 * 202 Accepted if the delete action is accepted but not enacted
                 * 204 No Content if the delete action has been enacted but the response does not include the deleted entity(s)
                return Request.CreateResponse(HttpStatusCode.NoContent);
                return Request.CreateResponse(HttpStatusCode.Accepted);
                */
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }

        // DELETE /api/products/?id=5&id=6&id=7
        // FromUri attribute is mandatory for MVC4 RC because the parameter is a complex type
        // and by default is taken from the request body
        public HttpResponseMessage Delete([FromUri]IEnumerable<int> index)
        {
            var entities = new ProductsEntities();
            List<MyProduct> deletedProducts = new List<MyProduct>();
            if (ModelState.IsValid)
            {
                foreach (int i in index)
                {
                    MyProduct deletedProduct = new MyProduct();
                    Product entity = entities.Products.Where(p => p.ID == i).First();
                    deletedProduct.ID = entity.ID;
                    deletedProduct.Name = entity.Name;
                    deletedProduct.Price = entity.Price;
                    deletedProduct.Quantity = entity.Quantity;
                    deletedProduct.ReleaseDate = entity.ReleaseDate;
                    entities.Products.DeleteObject(entity);
                    deletedProducts.Add(deletedProduct);
                }
                try
                {
                    entities.SaveChanges();
                }
                catch
                {
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));
                }
                return Request.CreateResponse(HttpStatusCode.OK, deletedProducts);
                /* Alternatively Delete can also return 
                 * 202 Accepted if the delete action is accepted but not enacted
                 * 204 No Content if the delete action has been enacted but the response does not include the deleted entity(s)
                return Request.CreateResponse(HttpStatusCode.NoContent);
                return Request.CreateResponse(HttpStatusCode.Accepted);
                */
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion
    }
}